@@ -30,14 +30,13 @@
 #' columns of the distance matrix.
 #'
 #' The other option is to provide a string. The string can represent a compatible registered distance of
-#' \code{\link[proxy]{dist}}. In the case of hierarchical algorithms, extra parameters can be provided in
-#' \code{...}.
+#' \code{\link[proxy]{dist}}. Extra parameters can be provided in \code{...}. See the examples.
 #'
 #' Additionally, with either type of algorithm, it can be one of the following custom implementations:
 #'
 #' \itemize{
-#'   \item \code{"dtw"}: DTW with L1 norm and optionally a Sakoe-Chiba constraint.
-#'   \item \code{"dtw2"}: DTW with L2 norm and optionally a Sakoe-Chiba constraint.
+#'   \item \code{"dtw"}: DTW with L1 norm and optionally a Sakoe-Chiba/Slanted-band constraint.
+#'   \item \code{"dtw2"}: DTW with L2 norm and optionally a Sakoe-Chiba/Slanted-band constraint.
 #'   \item \code{"dtw_lb"}: DTW with L1 or L2 norm and optionally a Sakoe-Chiba constraint. Some computations
 #'   are avoided by first estimating the distance matrix with Lemire's lower bound and then iteratively
 #'   refining with DTW. See \code{\link{dtw_lb}}.
@@ -59,7 +58,7 @@
 #'
 #' \itemize{
 #'   \item For matrix input, it will receive a matrix as single input. Each row will be a series that belongs
-#'   to a given cluster. The function should return a numeric vector representing the centroid series.
+#'   to a given cluster. The function should return a numeric vector with the centroid time series.
 #'   \item For a list input, the function will receive three inputs in the following order: the \emph{whole}
 #'   data list; a numeric vector with length equal to the number of series in \code{data}, indicating which
 #'   cluster a series belongs to; the current number of total clusters.
@@ -74,8 +73,8 @@
 #'   \item \code{"shape"}: Shape averaging. See \code{\link{shape_extraction}} for more details.
 #'   \item \code{"dba"}: DTW Barycenter Averaging. See \code{\link{DBA}} for more details.
 #'   \item \code{"pam"}: Partition around medoids. This basically means that the cluster centers are always
-#'   one of the time series in the data. In this case, the distance matrix is pre-computed once using all
-#'   time series in the data and then re-used at each iteration.
+#'   one of the time series in the data. In this case, the distance matrix can be pre-computed once using all
+#'   time series in the data and then re-used at each iteration. It usually saves overhead overall.
 #' }
 #'
 #' Note that only \code{dba} and \code{pam} support series of different lengths
@@ -105,12 +104,6 @@
 #'
 #' @section Notes:
 #'
-#' In order to ensure that the parameter values are detected correctly by the included functions
-#' when partitional clustering is used, the
-#' environment of the \code{dtwclust} function is assigned as an attribute of \code{data} via
-#' \code{attr(data, "env")} \code{<-} \code{environment()}. If the user alters the dataset with a
-#' preprocessing function, it should make sure that this attribute is maintained.
-#'
 #' Notice that the lower bounds are defined only for time series of equal lengths. \code{DTW} and \code{DTW2}
 #' don't require this, but they are much slower to compute.
 #'
@@ -136,52 +129,77 @@
 #'
 #' @examples
 #'
-#' # Load data
+#' #### Load data
 #' data(uciCT)
 #'
 #' # Reinterpolate to same length and coerce as matrix
-#' data <- t(sapply(CharTraj, reinterpolate, newLength = 205))
+#' data <- t(sapply(CharTraj, reinterpolate, newLength = 180))
 #'
-#' # Simple partitional clustering with L2 distance and PAM
-#' kc.l2 <- dtwclust(data, k = 20, distance = "L2", centroid = "pam",
-#'                   seed = 3247, trace = TRUE, save.data = TRUE)
-#' cat("Rand index for L2+PAM:", randIndex(kc.l2, CharTrajLabels), "\n\n")
+#' # Subset for speed
+#' data <- data[1:20, ]
+#' labels <- CharTrajLabels[1:20]
+#'
+#' #### Simple partitional clustering with L2 distance and PAM
+#' kc.l2 <- dtwclust(data, k = 4, distance = "L2", centroid = "pam",
+#'                   seed = 3247, trace = TRUE)
+#' cat("Rand index for L2+PAM:", randIndex(kc.l2, labels), "\n\n")
+#'
+#' #### TADPole clustering
+#' kc.tadp <- dtwclust(data, type = "tadpole", k = 4,
+#'                     window.size = 20, dc = 1.5,
+#'                     trace = TRUE)
+#' cat("Rand index for TADPole:", randIndex(kc.tadp, labels), "\n\n")
+#' plot(kc.tadp)
+#'
+#' # Modify plot
+#' plot(kc.tadp, cl = 1:2, labs.arg = list(title = "TADPole, clusters 1 and 2",
+#'                                         x = "time", y = "series"))
+#'
+#' #### Registering a custom distance with the 'proxy' package and using it
+#' # Normalized DTW distance
+#' ndtw <- function(x, y, ...) {
+#'   dtw::dtw(x, y, step.pattern = symmetric2,
+#'            distance.only = TRUE, ...)$normalizedDistance
+#' }
+#'
+#' # Registering the function with 'proxy'
+#' proxy::pr_DB$set_entry(FUN = ndtw, names=c("nDTW"),
+#'                        loop = TRUE, type = "metric", distance = TRUE,
+#'                        description = "Normalized DTW with L1 norm")
+#'
+#' # Subset of (original) data for speed
+#' # Change pam.precompute to FALSE to see time difference
+#' kc.ndtw <- dtwclust(CharTraj[31:40], distance = "nDTW",
+#'                     trace = TRUE, pam.precompute = TRUE,
+#'                     seed = 8319)
+#' cat("Rand index for nDTW (subset):",
+#'     randIndex(kc.ndtw, CharTrajLabels[31:40]), "\n\n")
+#' plot(kc.ndtw)
+#'
+#' #### Hierarchical clustering based on shabe-based distance (different lengths)
+#' hc.sbd <- dtwclust(CharTraj, type = "hierarchical",
+#'                    distance = "sbd", trace = TRUE)
+#' cl.sbd <- cutree(hc.sbd, 20)
+#' cat("Rand index for HC+SBD:", randIndex(cl.sbd, CharTrajLabels), "\n\n")
 #'
 #' \dontrun{
-#' # Saving and modifying the ggplot object with custom time
+#' #### Saving and modifying the ggplot object with custom time
 #' t <- seq(Sys.Date(), len = 205, by = "day")
 #' gkc <- plot(kc.l2, time = t, plot = FALSE)
 #'
 #' require(scales)
 #' gkc + scale_x_date(labels = date_format("%b-%Y"),
 #'                    breaks = date_breaks("2 months"))
-#' }
 #'
-#' # TADPole clustering (takes around 5 seconds)
-#' kc.tadp <- dtwclust(data, type = "tadpole", k = 20,
-#'                     window.size = 20, dc = 1.5, save.data = TRUE)
-#' cat("Rand index for TADPole:", randIndex(kc.tadp, CharTrajLabels), "\n\n")
-#' plot(kc.tadp)
+#' #### Use full DTW and PAM (takes around two minutes)
+#' kc.dtw <- dtwclust(CharTraj, k = 20, seed = 3251, trace = TRUE)
 #'
-#' # Modify plot
-#' plot(kc.tadp, cl = 1:4, labs.arg = list(title = "TADPole, clusters 1 through 4",
-#'                                         x = "time", y = "series"))
+#' #### Use full DTW with DBA centroids (takes around five minutes)
+#' kc.dba <- dtwclust(CharTraj, k = 20, centroid = "dba", seed = 3251, trace = TRUE)
 #'
-#' \dontrun{
-#' # Hierarchical clustering based on shabe-based distance
-#' hc.sbd <- dtwclust(data, type = "hierarchical", distance = "sbd")
-#' cl.sbd <- cutree(hc.sbd, 20)
-#' cat("Rand index for HC+SBD:", randIndex(cl.sbd, CharTrajLabels), "\n\n")
-#'
-#' # Use full DTW and PAM (takes around two minutes)
-#' kc.dtw <- dtwclust(data, k = 20, seed = 3251, trace = TRUE)
-#'
-#' # Use full DTW with DBA centroids (takes around five minutes)
-#' kc.dba <- dtwclust(data, k = 20, centroid = "dba", seed = 3251, trace = TRUE)
-#'
-#' # Use constrained DTW with original series of different lengths (around one minute)
+#' #### Use constrained DTW with original series of different lengths (around one minute)
 #' kc.cdtw <- dtwclust(CharTraj, k = 20, window.size = 20,
-#'                     seed = 3251, trace = TRUE, save.data = TRUE)
+#'                     seed = 3251, trace = TRUE)
 #'
 #' # Plot one of the clusters
 #' plot(kc.cdtw, cl=18)
@@ -212,6 +230,8 @@
 #' \code{distance = "DTW2"} (which always uses \code{L2}).
 #' @param dc Cutoff distance for TADPole algorithm.
 #' @param dba.iter Maximum number of iterations for \code{\link{DBA}} centroids.
+#' @param pam.precompute Precompute the whole distance matrix once and reuse it at each iteration if using PAM
+#' centroids. Otherwise calculate distances at every iteration.
 #' @param control Parameters for partitional clustering algorithms. See
 #' \code{\link[flexclust]{flexclustControl}}.
 #' @param save.data Return a copy of the data in the returned object? Ignored for hierarchical clustering.
@@ -232,8 +252,8 @@
 dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average",
                      distance = "dtw", centroid = "pam", preproc = NULL,
                      window.size = NULL, norm = "L1", dc = NULL,
-                     dba.iter = 50, control = NULL, save.data = FALSE,
-                     seed = NULL, trace = FALSE,
+                     dba.iter = 50, pam.precompute = TRUE, control = NULL,
+                     save.data = TRUE, seed = NULL, trace = FALSE,
                      ...)
 {
      ## =================================================================================================================
@@ -257,10 +277,6 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
           if (k < 2)
                stop("At least two clusters must be defined")
 
-          ## Used by some of the custom functions so that they know where to look for parameters
-          ## This is done automatically due to lexical scoping, but I rather do it explicitly
-          attr(data, "env") <- environment()
-
           if (is.function(centroid)) {
                cent <- centroid
 
@@ -289,31 +305,46 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
                                     cent = cent)
 
           } else if (is.character(distance)) {
-               family <- kccaFamilies(distance, cent, window.size, norm) # utils.R
+               # utils.R
+               family <- kccaFamilies(distance = distance,
+                                      cent = cent,
+                                      window.size = window.size,
+                                      norm = norm,
+                                      distmat = NULL,
+                                      ...)
 
           } else {
-               stop("Unspported distance definition")
+               stop("Unsupported distance definition")
           }
 
           ## ----------------------------------------------------------------------------------------------------------
           ## Replace specific functions if necessary
           ## ----------------------------------------------------------------------------------------------------------
 
-          distmat <- NULL
+          distmat <- NULL # replaced if appropriate
 
+          # replace with functions from utils.R
           if (is.character(centroid) && centroid == "shape") {
-               family@allcent <- allcent_se
+               family@allcent <- allcent_se() # closure, utils.R
 
                family@preproc <- preproc_se
 
           } else if (is.character(centroid) && centroid == "dba") {
-               family@allcent <- allcent_dba
+               family@allcent <- allcent_dba(dba.iter, window.size, norm, trace) # closure, utils.R
 
           } else if (is.character(centroid) && centroid == "pam") {
-               family@allcent <- allcent_pam
+               if (pam.precompute)
+                    distmat <- distmat_pam(data, family) # utils.R
 
-               distmat <- distmat_pam(data, family)
+               ## Redefine family with new distmat (to update closures)
+               family <- kccaFamilies(distance = distance,
+                                      cent = cent,
+                                      window.size = window.size,
+                                      norm = norm,
+                                      distmat = distmat,
+                                      ...)
 
+               family@allcent <- allcent_pam(distmat, family@dist) # another closure, utils.R
           }
 
           if (!is.null(preproc)) {
@@ -323,7 +354,7 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
                     stop("Invalid preprocessing")
 
           } else if (is.character(centroid) && centroid == "shape") {
-               preproc <- "zscore"
+               preproc <- "zscore" # string, just to indicate what was used...
 
           } else {
                preproc <- "none"
@@ -352,24 +383,46 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
                lengths <- sapply(data, length)
 
                if (length(unique(lengths)) > 1) {
-                    if (is.character(distance) && !(distance %in% c("dtw", "dtw2", "sbd")))
-                         stop("Only the following distances are supported for series of different lengths:\n
-                              \tdtw \tdtw2 \tsbd")
+                    if (is.character(distance) &&
+                        (distance %in% c("dtw", "dtw2", "dtw_lb", "lbk", "lbi", "sbd")) && # restrict check to these
+                        !(distance %in% c("dtw", "dtw2", "sbd")))
+                         stop("Only the following distances are supported for series of different lengths:\n\tdtw \tdtw2 \tsbd")
 
-                    if (is.character(centroid) && !(centroid %in% c("dba", "pam")))
-                         stop("Only the following centroids are supported for series of different lengths:\n
-                              \tdba \tpam")
+                    if (is.character(centroid) &&
+                        (centroid %in%  c("mean", "median", "shape", "dba", "pam")) && # restrict check to these
+                        !(centroid %in% c("dba", "pam")))
+                         stop("Only the following centroids are supported for series of different lengths:\n\tdba \tpam")
                }
 
-               if (is.function(centroid))
+               ## Replace allcent function with an appropriate one, either user provided or a custom for mean/median
+               if (is.function(centroid)) {
                     family@allcent <- centroid
 
+               } else if(is.character(centroid) && centroid == "mean") {
+                    family@allcent <- function(x, cluster, k) {
+                         X <- split(x, cluster)
+                         X <- lapply(X, function(l) t(sapply(l, rbind)))
+
+                         centers <- lapply(X, colMeans)
+                    }
+
+               } else if (is.character(centroid) && centroid == "median") {
+                    family@allcent <- function(x, cluster, k) {
+                         X <- split(x, cluster)
+                         X <- lapply(X, function(l) t(sapply(l, rbind)))
+
+                         centers <- lapply(X, function(x) apply(x, 2, stats::median))
+                    }
+               }
+
+               ## Cluster
                kc <- kcca.list(x = data,
                                k = k,
                                family = family,
                                control = ctrl)
 
           } else {
+               ## Cluster
                kc <- flexclust::kcca(x = data,
                                      k = k,
                                      family = family,
@@ -383,6 +436,7 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
           ## ----------------------------------------------------------------------------------------------------------
 
           toc <- proc.time() - tic
+          class(toc) <- "numeric"
 
           if (save.data)
                datalist <- consistency_check(data, "tsmat")
@@ -394,7 +448,8 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
                       distance = ifelse(is.function(distance), as.character(substitute(distance))[1], distance),
                       centroid = ifelse(is.function(centroid), as.character(substitute(centroid))[1], centroid),
                       preproc = ifelse(is.function(preproc), as.character(substitute(preproc))[1], preproc),
-                      datalist = datalist)
+                      datalist = datalist,
+                      proctime = toc)
 
           if (trace)
                cat("\n\tElapsed time is", toc["elapsed"], "seconds.\n\n")
@@ -415,9 +470,10 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
           lengths <- sapply(x, length)
 
           if (length(unique(lengths)) > 1) {
-               if (!is.function(distance) && !(distance %in% c("dtw", "dtw2", "sbd")))
-                    stop("Only the following distances are supported for series of different lengths:\n
-                         \tdtw \tdtw2 \tsbd")
+               if (is.character(distance) &&
+                   (distance %in% c("dtw", "dtw2", "dtw_lb", "lbk", "lbi", "sbd")) && # restrict check to these
+                   !(distance %in% c("dtw", "dtw2", "sbd")))
+                    stop("Only the following distances are supported for series of different lengths:\n\tdtw \tdtw2 \tsbd")
           }
 
           if (!is.null(preproc) && is.function(preproc)) {
@@ -466,7 +522,7 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
                            dtw_lb = {
                                 window.size <- consistency_check(window.size, "window")
 
-                                dtw_lb(x, x, window.size, norm = norm, error.check=TRUE)
+                                dtw_lb(x, x, window.size, norm = norm, error.check = TRUE)
                            },
 
 
@@ -476,7 +532,7 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
 
                                 proxy::dist(x = x, y = x,
                                             method = "LBI", window.size = window.size, norm = norm,
-                                            force.symmetry = TRUE, error.check=TRUE,
+                                            force.symmetry = TRUE, error.check = TRUE,
                                             ...)
                            },
 
@@ -495,7 +551,7 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
                                 x <- lapply(x, zscore)
 
                                 proxy::dist(x = x, y = x,
-                                            method = "SBD",
+                                            method = "SBD", error.check = TRUE,
                                             ...)
                            },
 
@@ -512,14 +568,16 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
                cat("\n\tPerforming hierarchical clustering...\n")
 
           ## Required form for 'hclust'
-          D <- D[lower.tri(D)]
+          DD <- D[lower.tri(D)]
 
           ## Needed attribute for 'hclust' (case sensitive)
-          attr(D, "Size") <- length(x)
+          attr(DD, "Size") <- length(x)
+          attr(DD, "method") <- attr(D, "method")
 
-          hc <- stats::hclust(D, method = method)
+          hc <- stats::hclust(DD, method = method)
 
           toc <- proc.time() - tic
+          class(toc) <- "numeric"
 
           if (trace)
                cat("\n\tElapsed time is", toc["elapsed"], "seconds.\n\n")
@@ -554,9 +612,17 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
                preproc <- "none"
           }
 
-          x <- consistency_check(data, "tsmat")
+          if (is.list(data)) {
+               consistency_check(data, "tslist")
+               x <- data
+               data <- t(sapply(data, rbind))
 
-          consistency_check(x, "tslist")
+          } else if (is.matrix(data)) {
+               x <- consistency_check(data, "tsmat")
+
+          } else {
+               stop("Unsupported format for data")
+          }
 
           ## ----------------------------------------------------------------------------------------------------------
           ## Cluster
@@ -569,7 +635,7 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
 
           if (trace) {
                cat("\nTADPole completed, pruning percentage = ",
-                   formatC(100-R$distCalcPercentage, format = "fg", digits = 3),
+                   formatC(100-R$distCalcPercentage, digits = 3, width = -1, format = "fg"),
                    "%\n",
                    sep = "")
           }
@@ -578,15 +644,19 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
           ## Prepare results
           ## ----------------------------------------------------------------------------------------------------------
 
+          toc <- proc.time() - tic
+          class(toc) <- "numeric"
+
           if (save.data) {
                tadpc <- new("dtwclust",
                             type = type,
                             distance = "DTW2",
                             centroid = "TADPole (PAM)",
                             preproc = ifelse(is.function(preproc), as.character(substitute(preproc))[1], preproc),
+                            proctime = toc,
 
                             call = MYCALL,
-                            centers = data[R$centers, ],
+                            centers = data[R$centers, , drop = FALSE],
                             k = as.integer(k),
                             cluster = as.integer(R$cl),
                             data = modeltools::ModelEnvMatrix(designMatrix = data),
@@ -597,15 +667,14 @@ dtwclust <- function(data = NULL, type = "partitional", k = 2, method = "average
                             distance = "DTW2",
                             centroid = "TADPole (PAM)",
                             preproc = ifelse(is.function(preproc), as.character(substitute(preproc))[1], preproc),
+                            proctime = toc,
 
                             call = MYCALL,
-                            centers = data[R$centers, ],
+                            centers = data[R$centers, , drop = FALSE],
                             k = as.integer(k),
                             cluster = as.integer(R$cl))
           }
 
-          toc <- proc.time() - tic
-
           if (trace)
                cat("\n\tElapsed time is", toc["elapsed"], "seconds.\n\n")
 
